/*
 * This code is a reference design for rotational flow meter with 2 LC sensors
 * An I2C cable is used to link up the Main board and motor board.
 * An Auto process to will be started once the boards are powered up.
 * The main board will undergo Auto TSM setting, and optimize the reference voltage setting for DAC.
 *
 * The number of rotation  detected by ESI will be displayed in the lower digits of the LCD,
 * while the upper digits will display the rotation number from motor board. The data is sent via the I2C.
 *
 * When measuring the current consumption, remove all jumpers of the main board except for the jumper of GND above J401.
 * and tap the current meter to the jumper 3V3.
 * To measure the current consumption of ESI, without LCD and I2C,
 * press the black button on the main board and remove the I2C cable.
 *
 * To turn on the LCD again, press the black button to toggle it.
 *
 * Texas Instruments
 *
 * Author: Thomas Kot and M. Stefanov
 * Date  : July 2014, April 2015
 */


#include "msp430fr6989.h"
#include "LCD.h"
#include "ScanIF.h"
#include "ESI_ESIOSC.h"
#include "IIC.h"

#include <stdint.h>
#include "mbus_packet.h"
#include "cc112x_wmbus868.h"
#include "chip_detect.h"
#include "cc112x_spi.h"

#define PKTLEN       20 // 1 < PKTLEN < 126 , 20 Bytes for Payload

/*******************************************************************************
*               Packet Examples from EN13757-4:2012
*   Packet S2:  0F 44 AE 0C 78 56 34 12 01 07 44 47 78 0B 13 43 65 87 1E 6D     (20 Bytes)
*   Packet C2:  14 44 AE 0C 78 56 34 12 01 07 8C 20 27 78 0B 13 43 65 87 7A C5  (21 Bytes)
*/
uint8_t wMBusPacketST[]= {0x0F, PACKET_C_FIELD, 0xAE, 0x0C, PACKET_CI_FIELD, 0x56, 0x34, 0x12, MAN_ID, MAN_VER, 0x44, 0x47, PACKET_CI_FIELD, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x1E, 0x6D}; //S2-Mode
uint8_t wMBusPacketC[]= {0x14,PACKET_C_FIELD, 0xAE, 0x0C, PACKET_CI_FIELD, 0x56, 0x34, 0x12, MAN_ID, MAN_VER, 0x8C, 0x20, 0x27, PACKET_CI_FIELD, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x7A, 0xC5}; //C2-Mode

//char TXpacket[291];
//uint8_t TXdataLength = 5;

extern const uint8_t WMbus_mode;
extern const uint8_t SKY_PA_onboard;
uint8_t   writeByte=0;
uint16_t i=0;
unsigned char test=0;
static radioChipType_t pRadioChipType;
volatile unsigned int * rf_device = 0;                   // RF device Type: CC1120, CC1200, ...
volatile unsigned char * chip_ver = 0;      	  // RF Chip version 2.1 or 2.3, ...
volatile uint16_t txPackets = 0;

_Bool bCC1120_Send_Flag = 0;
extern uint16_t  packetLength, TmodeByteLength;
extern uint8_t TXpacket[];
extern uint8_t TXbytes[];
extern uint8_t TXdataLength;
uint16_t ui16wmbusACC_Counter = 0; //8-bit field with increasing Number for AES-128, check EN13757-4 or OMSv3/v4 spec
uint8_t ui8packetTxSemaphore = 0, seconds = 0;

#define Time_out  8192      					// 2 sec for time out of Recalibration
#define Time_to_Recal 8192  					// 2 sec for testing use, 40960 for 10 sec

extern 	unsigned char  Status_flag;
volatile uint8_t read_reg;
extern uint8_t testmode_cw;
uint8_t testmode_cw_flag = 0;
char Power_measure = 0;
signed int  test_status = 0;
unsigned char ReCal_Flag ;
unsigned char RTCTimeoutFlag = 0;
int  rotation_counter = 0;

unsigned int Record_INT1=0;
unsigned int Record_INT2=0;
unsigned int Record_INT3=0;
unsigned int Record_INT4=0;


void Set_Clock(void);
void Port_Init(void);
void Set_Timer_A(void);
void Check_debug(void);
void Disable_all_IE(void);
void Enable_all_IE(void);

extern RegisterConfig(void);
extern rfStatus_t TrxSpiCmdStrobe(uint8_t cmd);
void wmbusInit();

void Port_Init()	{

	/*
	 * This is to set the I/O port in the initialization.
	 * The power consumption will be 0.8uA in LPM4 after setting.
	 *
	 */

	P1DIR=0xFF;
	P1OUT=0;

	P1DIR &= ~BIT2 + ~BIT1 + ~BIT0;		//GDO0 and GDO2
	P1REN |= BIT2 + BIT1 + BIT0;	//Input selected
	P1OUT |= BIT2 + BIT1 + BIT0; //Pull-up selected


	P2DIR=0xFF;
	P2OUT=0;
	P3DIR=0xFF;
	P3OUT=0;
	P4DIR=0xFF;
	P4OUT=0;
	P5DIR=0xFF;
	P5OUT=0;
	P6DIR=0xFF;
	P6OUT=0;
	P7DIR=0xFF;
	P7OUT=0;
	P8DIR=0xFF;
	P8OUT=0;
	P9DIR=0x00;
	P9OUT=0;
	P10DIR=0xFF;
	P10OUT=0;
	PJDIR=0xFF;
	PJOUT=0;

	CECTL3 = 0xFF00;

	PM5CTL0 &= ~LOCKLPM5;;        				// In initialization, the I/Os are configured before unlocking the I/O ports

	PMMCTL0_H = 0xA5;
//	PMMCTL0_L |= PMMREGOFF;
	PMMCTL0_L &= ~SVSHE;
	PMMCTL0_H = 0xEE;
}


void Set_Clock(void)
{
// use external 32KHz crystal
// MClock and SMClock from DCO 4MHz

      PJSEL0 |=  BIT4 + BIT5;					// using LFXTCLK
	  CSCTL0 = 0XA500;                          // Write in the password for system clock setting
	  CSCTL1 = 0x0006;                          // 4MHz for DCO as DCORSEL=0
//	  CSCTL1	= DCOFSEL_3 + DCORSEL;		    // DCO = 8MHz because DCORSEL=1

	  CSCTL2 = 0x0033;                          // ACLK from LFXTCLK, SMCLK and MCLK from DCO
	  CSCTL3 = 0x0000;							// ACLK div by 1, SMCLK div by 1, MCLK div by 1
	  CSCTL4 = 0x0148;							// HFXTOFF, LFXTDRIVE = 1, VLO off, SMCLK on

	  do {
	  	    CSCTL5 &= ~LFXTOFFG;                // Clear XT1 fault flag
	  	    SFRIFG1 &= ~OFIFG;
	  } while (SFRIFG1&OFIFG);               // Test oscillator fault flag

	  CSCTL0_H = 0;  //Lock CS
}

/*
 *  Example RTC in clock mode
 *
 *  // Configure LFXT 32kHz crystal
    CSCTL0_H = CSKEY >> 8;                  // Unlock CS registers
    CSCTL4 &= ~LFXTOFF;                     // Enable LFXT

    */                          // Lock CS registers
void Set_RTC(void) {
    // Configure RTC_C
    RTCCTL0_H = RTCKEY_H;                   // Unlock RTC
    RTCCTL0_L = RTCTEVIE | RTCRDYIE;        // enable RTC read ready interrupt
                                            // enable RTC time event interrupt

    RTCCTL1 = RTCBCD | RTCHOLD | RTCMODE;   // RTC enable, BCD mode, RTC hold

    RTCYEAR = 0x2015;                       // Year = 0x2015
    RTCMON = 0x04;                           // Month = 0x04 = April
    RTCDAY = 0x15;                          // Day = 0x15 = 15th
    RTCDOW = 0x01;                          // Day of week = 0x01 = Monday
    RTCHOUR = 0x10;                         // Hour = 0x10
    RTCMIN = 0x32;                          // Minute = 0x32
    RTCSEC = 0x45;                          // Seconds = 0x45

    RTCADOWDAY = 0x2;                       // RTC Day of week alarm = 0x2
    RTCADAY = 0x20;                         // RTC Day Alarm = 0x20
    RTCAHOUR = 0x10;                        // RTC Hour Alarm
    RTCAMIN = 0x23;                         // RTC Minute Alarm

    RTCCTL1 &= ~(RTCHOLD);                  // Start RTC

	RTCCTL0_L	&= ~RTCOFIFG;		// Clear XTAL error flag
	RTCCTL0_H	= 0xff;				// Lock RTC

    __bis_SR_register(LPM3_bits | GIE);     // Enter LPM3 mode w/ interrupts enabled
   // __no_operation();
}

void Init_RTC(void)
{
	// RTC
	RTCCTL0_H	= RTCKEY_H;		// Unlock RTC
	RTCCTL0_L	|= RTCTEVIE;	// Enable RTC timer event interrupt

	// RTCBCD	= 0		N/A
	// RTCHOLD	= 1		RTC is held
	// RTCMODE	= 0		Counter mode
	// RTCRDY	= 0		N/A
	// RTCSSELx = 2		Clock sourced from RT1PS
	// RTCEVx	= 0		Counter mode: 8 bit overflow
	RTCCTL1 = RTCSSEL_2 + RTCTEV_0 + RTCHOLD;

	// RTC Calibration not set
	// RTCCTL3 = xxx
	// RTCOCAL = xxx

	// RTC temperature compensation not set
	// RTCTCMP = xxx

	// Set Pre-scaler
	// 32.768kHz/128(div) = 256(8 bit overflow)
	// RT1PSDIV = 6		Divided by 128
	RTCPS1CTL = RT1PSDIV_6;

	RTCCTL1		&= ~RTCHOLD;		// Enable RTC
	RTCCTL0_L	&= ~RTCOFIFG;		// Clear XTAL error flag
	RTCCTL0_H	= 0xff;				// Lock RTC
}


void Set_Timer_A(void)
{
/*  This is the timer for triggering the run time re-calibration
 *  2 sec is set for demonstration purpose
 *  The actual timer setting depends on the application.
 */

	TA0CTL = TASSEL0 + ID0 + ID1 + TACLR; 		//Aclk divided by 8;
	TA0CCR0 = Time_to_Recal;              		// 2 sec for testing use
	TA0CCTL0 = CCIE; 					  		// INT enable, INT flag cleared
//	TA0CTL |= MC0;
}


void Disable_all_IE(void)
{
	Record_INT1 =  ESIINT1;
	Record_INT2 = (TA0CCTL0&CCIE) ;
	Record_INT3 = (UCB0IE &(UCTXIE | UCRXIE  | UCSTPIE | UCNACKIE));

	ESIINT1 = 0x0000;
	TA0CCTL0 &= ~CCIE;
	UCB0IE   &= ~(UCTXIE | UCRXIE  | UCSTPIE | UCNACKIE);

}

void Enable_all_IE(void)
{
   ESIINT1 = Record_INT1;
   TA0CCTL0 |= Record_INT2;
   UCB0IE   |= Record_INT3;

}

void main(void)
{
	WDTCTL = WDTPW + WDTHOLD;					// disable Watchdog

	Port_Init();
	Set_Clock();

	P1IES |= BIT2;					// Set P1.2 as key input
	P1IFG &= ~BIT2;					// User can press the black button to toggle switch on/off the LCD
	P1IE  |= BIT2;

	P1IES |= BIT1;					// Set GPIO_HIGH_TO_LOW_TRANSITION
	P1IFG &= ~BIT1;					// Clear Flag
	P1IE  |= BIT1;

	P1IES |= BIT0;					// Set GPIO_HIGH_TO_LOW_TRANSITION
	P1IFG &= ~BIT0;					// Clear Flag
	P1IE  |= BIT0;


	P1OUT |= BIT6 + BIT7; 						// for I2C pull up. For measuring current consumption, remove the physical I2C cable.

//	__bis_SR_register(LPM4_bits+GIE);   		// un-remark this instruction. Engineer can measure the current consumption here. 0.7 to 0.8uA

	// LEDs on EVM430
    PJDIR = BIT2 + BIT3;
	PJOUT = 0;

	init_LCD();

	EsioscInit(ESIOSC_Default);       			// default setting = 4.8MHz for internal oscillator of ESI
	InitScanIF();								// Initialization of ScanIf module
	lcd_display_num(0,0);						// Display "0" on the lower digit which indicating the rotation number detected from ESI

	ReCal_Flag = 0;                 			// Init the status flag with non-runtime calibration

	TSM_Auto_cal();							// Auto set the TSM delay
	Find_Noise_level();						// To find the noise level detected by ESI

	lcd_display_num(8888,0);						// Display "8888" on LCD indicating the completion of TSM calibration.

	//
	Set_DAC();								// noise level found.
	Status_flag |= BIT2;					// indicating Calibration of DAC process completed
	ScanIF_PostInit();
	ReCal_Flag = 0;                 		// Init the status flag with non-runtime calibration
	// End of power up calibration

	// Set ESI Interrupts
	ESIINT1 &= ~ESIIE5;
	ESICTL  |= ESIEN;						// ESI enable. This will reset all counters of ESI. For actual operation of flowmeter, switch on ESI and will always on till battery drain off
	ESIINT2 &= ~ESIIFG5;					// clear INT flag of Q6 of PSM
	ESIINT1 |= ESIIE5;						// enable INT of Q6

	rotation_counter = ESICNT1;				// get the ESI counter for number of rotation
	lcd_display_num(rotation_counter/4, 0);	// to display the number of rotation from ESI in low digits of LCD

	Set_RTC();

	wmbusInit();    // initialize wMBus mode
	RegisterConfig();  //initialize CC1xxx device with Settings as selected per WMbus_mode in mbus_packet.h

	 while(1)	                					// Infinite loop for demonstration purpose
	 {
		 //wM-Bus TX operation

		 if (testmode_cw && testmode_cw_flag == 0) {
			 //set CC1120 in CW mode
			 writeByte =   0x06;
			 cc112xSpiWriteReg(CC112X_MDMCFG1,&writeByte,1);
			 writeByte =   0x01;
			 cc112xSpiWriteReg(CC112X_CFM_DATA_CFG,&writeByte,1);
			 //set TX power
			 if (SKY_PA_onboard) {

				 //Set Control lines from MSP430 to SKY_PA
				 //TX: all 3 lines = 1
				 //Havok P2.3=CTX Signal; P2.1=CSD_L and P2.0=CBYP_L
				 //P8OUT |= BIT4;	//Input selected
				// P2OUT |= BIT3 + BIT1 + BIT0;  //BIT0 =1 TX PA = Supply breaks down, USe EXT> VCC
				 //TX with Bypass: 1 1 0=CBYP_L
				 P2OUT |= BIT3 + BIT1; //This works for TPS62237 and breaks if PA is enabled
				 //RX: 1 0 X
				 //Sleep: CSD_L=0 X X

				// P2OUT |= BIT0;


					/* Read PA_CFG2 Register
				 cc112xSpiReadReg(CC112X_PA_CFG2, &read_reg, 1);
				 if (read_reg >= 0x5B) { //-4dBm limit for PA input signal
					 writeByte =   0x4D; //set to -4.5dbm
					 cc112xSpiWriteReg(CC112X_PA_CFG2,&writeByte,1);
				 }
				 */
			 }

			 testmode_cw_flag = 1;
			 // strobe TX to send packet
			 TrxSpiCmdStrobe(CC112X_STX);

			 //if button pressed Stop TestMode and go back to Packet mode

		 }

		 if (bCC1120_Send_Flag  && testmode_cw == 0) 		{ //TX operation based on RTC flag
						// update wMbus packet counter
			ui16wmbusACC_Counter++;
			RegisterConfig();  //initialize CC1xxx device with Settings as selected per WMbus_mode in mbus_packet.h

			// Calculate total number of bytes in the wireless MBUS packet
			//Update packet and UPDATE CRC16!!!
			if (WMbus_mode == WMBUS_SMODE || WMbus_mode == WMBUS_TMODE ) {
				//Update packet and UPDATE CRC16!!!
				// if Counter used, then CRC16 fails in T-mode  but works in S-mode!
				TXpacket[15] = (unsigned char) (rotation_counter/4 >> 16);
				TXpacket[16] = (unsigned char) (rotation_counter/4 >> 8);
				TXpacket[17] = (unsigned char) (rotation_counter/4);
			}

			if (WMbus_mode == WMBUS_SMODE) {
				encodeTXPacket(TXbytes, TXpacket, TXdataLength);
				packetLength = packetSize (TXpacket[0]); //works for S-mode packet with Manchester in HW
				cc112xSpiWriteTxFifo(TXbytes, packetLength);
			}

			else if (WMbus_mode == WMBUS_TMODE) {
				TXpacket[13] = DIF_FIELD; //0x0B; need overwrite due to TXpacket->TXbytes->TXpacket handling
				TXpacket[14] = VIF_FIELD; //0x13;
				encodeTXPacket(TXbytes, TXpacket, TXdataLength);

				//calculate T-mode TXpacket length
				packetLength = packetSize (TXbytes[0]); //works for S-mode packet with Manchester in HW
				encodeTXBytesTmode(TXpacket, TXbytes, packetLength);
				TmodeByteLength = byteSize(2, 1, packetLength);
				cc112xSpiWriteTxFifo(TXpacket, TmodeByteLength);
			}

			else if (WMbus_mode == WMBUS_CMODE || WMbus_mode == WMBUS_C2MODE || WMbus_mode == WMBUS_NabefMODE || WMbus_mode == WMBUS_NcdMODE || WMbus_mode == WMBUS_NgMODE ||
										 WMbus_mode == WMBUS_FMODE ) {
				wMBusPacketC[16] = (unsigned char) (rotation_counter/4 >> 16);
				wMBusPacketC[17] = (unsigned char) (rotation_counter/4 >> 8);
				wMBusPacketC[18] = (unsigned char) (rotation_counter/4);

				packetLength = packetSize (wMBusPacketC[0]);
				encodeTXPacket(TXbytes, wMBusPacketC, packetLength);

				// write wM-Bus packet to tx fifo
				cc112xSpiWriteTxFifo(TXbytes, packetLength);
			}

		   // strobe TX to send packet
		   TrxSpiCmdStrobe(CC112X_STX);

		   // wait for interrupt that packet has been sent.
		   // (Assumes the GPIO connected to the radioRxTxISR function is set
			// to GPIOx_CFG = 0x06)
		   while (!ui8packetTxSemaphore);
		   // clear semaphore flag
		   ui8packetTxSemaphore = 0;
		   txPackets++;
		 }

	#if AFE2_enable
		// Recalibration
		if(ReCal_Flag&BIT6)							// Check if Re-calibration flag is set
		{
			ReCal_Flag &= ~BIT7;                  	// Reset Bit7 for timer call

			TA0CTL	= TASSEL0 + ID__8 + MC_1+ TACLR; 		//Aclk divided by 8;
			TA0CCR0 = Time_out;                   			// set the time out timer. it will generate a time out interrupt when it stop rotating
			TA0CCTL0 = CCIE; 					  			// INT enable, INT flag cleared

			ESIINT1 &= ~ESIIE5;

			ReCalScanIF();           					// to do runtime calibration with AFE2

			TA0CTL  = 0;

			ReCal_Flag = 0;							// ReCal of AFE1 is done, reset all flags.

			ESIINT2 &= ~ESIIFG5;                  	// clear the Q6 flag
			ESIINT1 |= ESIIE5;						// Enable Q6 INT for in case of Time out.
		}
	#endif
		__bis_SR_register(LPM3_bits | GIE);   	// Enter into LPM3 and enable interrupts

	 }
}



#pragma vector=ESCAN_IF_VECTOR
__interrupt void ISR_ESCAN_IF(void)
{
	switch(__even_in_range(ESIIV, ESIIV_ESIIFG2))
	{
	case ESIIV_NONE: break;
	case ESIIV_ESIIFG1:
		_low_power_mode_off_on_exit();     								// exit low power mode;
		break;

	case ESIIV_ESIIFG0: break;
	case ESIIV_ESIIFG8: break;
	case ESIIV_ESIIFG3: break;

	case ESIIV_ESIIFG6: break;
	case ESIIV_ESIIFG5:
		if(ESIINT1&ESIIE5)
		{
			ESIINT2 &= ~ESIIFG5;                						// clear the Q6 flag

			if(Status_flag&BIT2)                						// Check for completion of Calibration of DAC
			{							    							// If yes, LCD is to display the rotation number
				if(!(ReCal_Flag&BIT6))
				{
					if (LCDCCTL0&LCDON)
					{
						rotation_counter = ESICNT1;									// get the ESI counter for number of rotation
						lcd_display_num(rotation_counter/4, 0);			// to display the number of rotation from ESI in low digits of LCD
					}
				}
				if (ReCal_Flag&BIT5)
				{
					_low_power_mode_off_on_exit();     					// exit low power mode;
				}
			}
		}
		break;

	case ESIIV_ESIIFG4: break;
	case ESIIV_ESIIFG7: break;
	case ESIIV_ESIIFG2: break;
	}
}


// Timer A0 interrupt service routine
#pragma vector = TIMER0_A0_VECTOR
__interrupt void Timer_A (void) {
    if (ReCal_Flag&BIT6)
    {
    	ReCal_Flag |= BIT1;                           								// Time out
    	ESIINT1 &= ~(ESIIE1+ESIIE5);				  								// Prevent entering ESI ISR to call an extra "_low_power_mode_off_on_exit()"
    	_low_power_mode_off_on_exit();       	      								// exit low power mode from ReCal_ScanIF ;
    }
	TA0CTL &= ~MC0;									 								// disable timer
}

void wmbusInit()
{
    // Set SPI to master 3-pin config
	SPI_Config();

	trxDetectChipType(&pRadioChipType);
	rf_device = (unsigned int*) pRadioChipType.deviceName;
	chip_ver = (unsigned char*) pRadioChipType.ver;

	if (rf_device == (unsigned int*) 0x1120 && chip_ver == (unsigned char*) 0x21) {
		ManualCalibration();
	}

}

#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(RTC_VECTOR))) RTC_ISR (void)
#else
#error Compiler not supported!
#endif
{

	switch(__even_in_range(RTCIV, RTCIV_RT1PSIFG))
		{
		case RTCIV_NONE:		break;
		case RTCIV_RTCOFIFG:	break;
		case RTCIV_RTCRDYIFG:
			//P1OUT ^= 0x01;                  // Toggles P1.0 every second
			        seconds++;
			       	PJOUT ^= BIT2;
			       	if (seconds == 5 ) {
			       		LPM3_EXIT;
			        		PJOUT ^= BIT3;
			        		bCC1120_Send_Flag = 1;  // TX the wM-Bus packet flag

			        		seconds = 0;
			        }
			       	break;
		case RTCIV_RTCTEVIFG:
			if(!(ReCal_Flag&BIT6))
			{
				RTCTimeoutFlag = 1;
				LPM3_EXIT;
			}
			break;
		case RTCIV_RTCAIFG:		break;
		case RTCIV_RT0PSIFG:	break;
		case RTCIV_RT1PSIFG:	break;
		}
}
/*
 * PORT1 Interrupt Service Routine
 * Handles S1 button press interrupt
 */
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
    //P1OUT |= BIT0;    // Turn LED1 On
    switch(__even_in_range(P1IV, P1IV_P1IFG7))
    {
        case P1IV_NONE : break;
        case P1IV_P1IFG0 : //Pin 1.0 is GPIO0, use as Interrupt Flag
        	// set packet semaphore
        			  ui8packetTxSemaphore = 1;
        	     //   else if ( *txrxSet == 2 /* RX operation selected */ &&  *wmbusRunning ==1)
        	      //    ui8packetRxSemaphore = 1;
        			// clear isr flag
        	break;
        case P1IV_P1IFG1 : //Pin 1.1 is GPIO2, use as Interrupt Flag

		// set packet semaphore Pin 1.5 is GPIO3, use as RX Sync word Interrupt Flag?
    //    if ( *txrxSet == 2 /* RX operation selected */ &&  *wmbusRunning ==1)
  //        ui8packetRxSemaphore = 1;
		// clear isr flag
        	break;
        case P1IV_P1IFG2 :
        	Power_measure ^= BIT0;

        if (Power_measure & BIT0)
        {LCDCCTL0 &= ~LCDON;
        ESIINT1 &= ~ESIIE5;																// disable INT of Q6
    	TA0CTL &= ~MC0;																	// stop the re-calibartion timer
        }
        else
        {LCDCCTL0 |= LCDON;
        ESIINT1 |= ESIIE5;																// enable INT of Q6
    	TA0CTL |= MC0;																	// turn on re-calibration timer
        }

        if (testmode_cw_flag) {  //End CW Test after first Push Button right
        	testmode_cw ^= 1;
        	testmode_cw_flag = 0;
        }
		   // strobe TX to send packet
		TrxSpiCmdStrobe(CC112X_SIDLE);
		writeByte =   0x00;
		cc112xSpiWriteReg(CC112X_CFM_DATA_CFG,&writeByte,1);

        ui8packetTxSemaphore=1;

       _low_power_mode_off_on_exit();

                break;

        case P1IV_P1IFG3 : break;
        case P1IV_P1IFG4 : break;
        case P1IV_P1IFG5 :

        	break;
        case P1IV_P1IFG6 : break;

        case P1IV_P1IFG7 : break;
    }
    P1OUT &= ~BIT0;    // Turn LED1 On
}

