/*****************************************************************************/
//  @file   CC112x_spi.h  
//  
//  @brief  header file that defines a minimum set of neccessary functions
//          to communicate with CC112X over SPI as well as the regsister 
//          mapping. 
//				 
//  Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/****************************************************************************/

#ifndef CC112x_SPI_H
#define CC112x_SPI_H

#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
 * INCLUDES
 */
#include <stdint.h>

#define SPI_PORT_SEL       P3SEL0
#define SPI_PORT_OUT       P3OUT
#define SPI_PORT_DIR       P3DIR
#define SPI_PORT_IN        P3IN

#define SPI_SPI_MOSI_PIN   BIT4   //UCA1 SPI Interface
#define SPI_SPI_MISO_PIN   BIT5
#define SPI_SPI_SCLK_PIN   BIT6
#define SPI_SPI_SC_N_PIN   BIT7

#define RF_RESET_N_PORT_OUT  P8OUT	//RF RESET
#define RF_RESET_N_PORT_SEL  P8SEL0
#define RF_RESET_N_PORT_DIR  P8DIR
#define RF_RESET_N_PIN       BIT7

#define CC_GDO0_PxOUT        P1OUT		//GDO0 is on P1.0
#define CC_GDO0_PxIN         P1IN
#define CC_GDO0_PxDIR        P1DIR
#define CC_GDO0_PxIE         P1IE
#define CC_GDO0_PxIES        P1IES
#define CC_GDO0_PxIFG        P1IFG
#define CC_GDO0_PIN          BIT0

#define CC_GDO2_PxOUT        P1OUT		//GDO2 is is on P1.1
#define CC_GDO2_PxIN         P1IN
#define CC_GDO2_PxDIR        P1DIR
#define CC_GDO2_PxIE         P1IE
#define CC_GDO2_PxIES        P1IES
#define CC_GDO2_PxIFG        P1IFG
#define CC_GDO2_PIN          BIT1

#define CC_GDO3_PxOUT        P2OUT		//GDO3 is is on P2.2
#define CC_GDO3_PxIN         P2IN
#define CC_GDO3_PxDIR        P2DIR
#define CC_GDO3_PxIE         P2IE
#define CC_GDO3_PxIES        P2IES
#define CC_GDO3_PxIFG        P2IFG
#define CC_GDO3_PIN          BIT2


#define RADIO_BURST_ACCESS   0x40
#define RADIO_SINGLE_ACCESS  0x00
#define RADIO_READ_ACCESS    0x80
#define RADIO_WRITE_ACCESS   0x00

extern _Bool bCC1120_Send_Flag;

typedef struct
{
  uint16_t  addr;
  uint8_t   data;
}registerSetting_t;

typedef uint8_t  rfStatus_t;

/******************************************************************************
 * CONSTANTS
 */

/* configuration registers */
#define CC112X_IOCFG3                   0x0000
#define CC112X_IOCFG2                   0x0001
#define CC112X_IOCFG1                   0x0002
#define CC112X_IOCFG0                   0x0003
#define CC112X_SYNC3                    0x0004
#define CC112X_SYNC2                    0x0005
#define CC112X_SYNC1                    0x0006
#define CC112X_SYNC0                    0x0007
#define CC112X_SYNC_CFG1                0x0008
#define CC112X_SYNC_CFG0                0x0009
#define CC112X_DEVIATION_M              0x000A
#define CC112X_MODCFG_DEV_E             0x000B
#define CC112X_DCFILT_CFG               0x000C
#define CC112X_PREAMBLE_CFG1            0x000D
#define CC112X_PREAMBLE_CFG0            0x000E
#define CC112X_FREQ_IF_CFG              0x000F
#define CC112X_IQIC                     0x0010
#define CC112X_CHAN_BW                  0x0011
#define CC112X_MDMCFG1                  0x0012
#define CC112X_MDMCFG0                  0x0013
#define CC112X_SYMBOL_RATE2             0x0014
#define CC112X_SYMBOL_RATE1             0x0015
#define CC112X_SYMBOL_RATE0             0x0016
#define CC112X_AGC_REF                  0x0017
#define CC112X_AGC_CS_THR               0x0018
#define CC112X_AGC_GAIN_ADJUST          0x0019
#define CC112X_AGC_CFG3                 0x001A
#define CC112X_AGC_CFG2                 0x001B
#define CC112X_AGC_CFG1                 0x001C
#define CC112X_AGC_CFG0                 0x001D
#define CC112X_FIFO_CFG                 0x001E
#define CC112X_DEV_ADDR                 0x001F
#define CC112X_SETTLING_CFG             0x0020
#define CC112X_FS_CFG                   0x0021
#define CC112X_WOR_CFG1                 0x0022
#define CC112X_WOR_CFG0                 0x0023
#define CC112X_WOR_EVENT0_MSB           0x0024
#define CC112X_WOR_EVENT0_LSB           0x0025
#define CC112X_PKT_CFG2                 0x0026
#define CC112X_PKT_CFG1                 0x0027
#define CC112X_PKT_CFG0                 0x0028
#define CC112X_RFEND_CFG1               0x0029
#define CC112X_RFEND_CFG0               0x002A
#define CC112X_PA_CFG2                  0x002B
#define CC112X_PA_CFG1                  0x002C
#define CC112X_PA_CFG0                  0x002D
#define CC112X_PKT_LEN                  0x002E

/* Extended Configuration Registers */
#define CC112X_IF_MIX_CFG               0x2F00
#define CC112X_FREQOFF_CFG              0x2F01
#define CC112X_TOC_CFG                  0x2F02
#define CC112X_MARC_SPARE               0x2F03
#define CC112X_ECG_CFG                  0x2F04
#define CC112X_CFM_DATA_CFG             0x2F05
#define CC112X_EXT_CTRL                 0x2F06
#define CC112X_RCCAL_FINE               0x2F07
#define CC112X_RCCAL_COARSE             0x2F08
#define CC112X_RCCAL_OFFSET             0x2F09
#define CC112X_FREQOFF1                 0x2F0A
#define CC112X_FREQOFF0                 0x2F0B
#define CC112X_FREQ2                    0x2F0C
#define CC112X_FREQ1                    0x2F0D
#define CC112X_FREQ0                    0x2F0E
#define CC112X_IF_ADC2                  0x2F0F
#define CC112X_IF_ADC1                  0x2F10
#define CC112X_IF_ADC0                  0x2F11
#define CC112X_FS_DIG1                  0x2F12
#define CC112X_FS_DIG0                  0x2F13
#define CC112X_FS_CAL3                  0x2F14
#define CC112X_FS_CAL2                  0x2F15
#define CC112X_FS_CAL1                  0x2F16
#define CC112X_FS_CAL0                  0x2F17
#define CC112X_FS_CHP                   0x2F18
#define CC112X_FS_DIVTWO                0x2F19
#define CC112X_FS_DSM1                  0x2F1A
#define CC112X_FS_DSM0                  0x2F1B
#define CC112X_FS_DVC1                  0x2F1C
#define CC112X_FS_DVC0                  0x2F1D
#define CC112X_FS_LBI                   0x2F1E
#define CC112X_FS_PFD                   0x2F1F
#define CC112X_FS_PRE                   0x2F20
#define CC112X_FS_REG_DIV_CML           0x2F21
#define CC112X_FS_SPARE                 0x2F22
#define CC112X_FS_VCO4                  0x2F23
#define CC112X_FS_VCO3                  0x2F24
#define CC112X_FS_VCO2                  0x2F25
#define CC112X_FS_VCO1                  0x2F26
#define CC112X_FS_VCO0                  0x2F27
#define CC112X_GBIAS6                   0x2F28
#define CC112X_GBIAS5                   0x2F29
#define CC112X_GBIAS4                   0x2F2A
#define CC112X_GBIAS3                   0x2F2B
#define CC112X_GBIAS2                   0x2F2C
#define CC112X_GBIAS1                   0x2F2D
#define CC112X_GBIAS0                   0x2F2E
#define CC112X_IFAMP                    0x2F2F
#define CC112X_LNA                      0x2F30
#define CC112X_RXMIX                    0x2F31
#define CC112X_XOSC5                    0x2F32
#define CC112X_XOSC4                    0x2F33
#define CC112X_XOSC3                    0x2F34
#define CC112X_XOSC2                    0x2F35
#define CC112X_XOSC1                    0x2F36
#define CC112X_XOSC0                    0x2F37
#define CC112X_ANALOG_SPARE             0x2F38
#define CC112X_PA_CFG3                  0x2F39
#define CC112X_IRQ0M                    0x2F3F
#define CC112X_IRQ0F                    0x2F40 

/* Status Registers */
#define CC112X_WOR_TIME1                0x2F64
#define CC112X_WOR_TIME0                0x2F65
#define CC112X_WOR_CAPTURE1             0x2F66
#define CC112X_WOR_CAPTURE0             0x2F67
#define CC112X_BIST                     0x2F68
#define CC112X_DCFILTOFFSET_I1          0x2F69
#define CC112X_DCFILTOFFSET_I0          0x2F6A
#define CC112X_DCFILTOFFSET_Q1          0x2F6B
#define CC112X_DCFILTOFFSET_Q0          0x2F6C
#define CC112X_IQIE_I1                  0x2F6D
#define CC112X_IQIE_I0                  0x2F6E
#define CC112X_IQIE_Q1                  0x2F6F
#define CC112X_IQIE_Q0                  0x2F70
#define CC112X_RSSI1                    0x2F71
#define CC112X_RSSI0                    0x2F72
#define CC112X_MARCSTATE                0x2F73
#define CC112X_LQI_VAL                  0x2F74
#define CC112X_PQT_SYNC_ERR             0x2F75
#define CC112X_DEM_STATUS               0x2F76
#define CC112X_FREQOFF_EST1             0x2F77
#define CC112X_FREQOFF_EST0             0x2F78
#define CC112X_AGC_GAIN3                0x2F79
#define CC112X_AGC_GAIN2                0x2F7A
#define CC112X_AGC_GAIN1                0x2F7B
#define CC112X_AGC_GAIN0                0x2F7C
#define CC112X_CFM_RX_DATA_OUT          0x2F7D
#define CC112X_CFM_TX_DATA_IN           0x2F7E
#define CC112X_ASK_SOFT_RX_DATA         0x2F7F
#define CC112X_RNDGEN                   0x2F80
#define CC112X_MAGN2                    0x2F81
#define CC112X_MAGN1                    0x2F82
#define CC112X_MAGN0                    0x2F83
#define CC112X_ANG1                     0x2F84
#define CC112X_ANG0                     0x2F85
#define CC112X_CHFILT_I2                0x2F86
#define CC112X_CHFILT_I1                0x2F87
#define CC112X_CHFILT_I0                0x2F88
#define CC112X_CHFILT_Q2                0x2F89
#define CC112X_CHFILT_Q1                0x2F8A
#define CC112X_CHFILT_Q0                0x2F8B
#define CC112X_GPIO_STATUS              0x2F8C
#define CC112X_FSCAL_CTRL               0x2F8D
#define CC112X_PHASE_ADJUST             0x2F8E
#define CC112X_PARTNUMBER               0x2F8F
#define CC112X_PARTVERSION              0x2F90
#define CC112X_SERIAL_STATUS            0x2F91
#define CC112X_MODEM_STATUS1            0x2F92
#define CC112X_MODEM_STATUS0            0x2F93
#define CC112X_MARC_STATUS1             0x2F94
#define CC112X_MARC_STATUS0             0x2F95
#define CC112X_PA_IFAMP_TEST            0x2F96
#define CC112X_FSRF_TEST                0x2F97
#define CC112X_PRE_TEST                 0x2F98
#define CC112X_PRE_OVR                  0x2F99
#define CC112X_ADC_TEST                 0x2F9A
#define CC112X_DVC_TEST                 0x2F9B
#define CC112X_ATEST                    0x2F9C
#define CC112X_ATEST_LVDS               0x2F9D
#define CC112X_ATEST_MODE               0x2F9E
#define CC112X_XOSC_TEST1               0x2F9F
#define CC112X_XOSC_TEST0               0x2FA0  
                                        
#define CC112X_RXFIRST                  0x2FD2   
#define CC112X_TXFIRST                  0x2FD3   
#define CC112X_RXLAST                   0x2FD4 
#define CC112X_TXLAST                   0x2FD5 
#define CC112X_NUM_TXBYTES              0x2FD6  /* Number of bytes in TXFIFO */ 
#define CC112X_NUM_RXBYTES              0x2FD7  /* Number of bytes in RXFIFO */
#define CC112X_FIFO_NUM_TXBYTES         0x2FD8  
#define CC112X_FIFO_NUM_RXBYTES         0x2FD9  


/*========================= REGISTER FIFOTHR =================================*/
/* Set the threshold for the TX FIFO and RX FIFO. The threshold is
   exceeded when the number of bytes in the FIFO is equal to or higher
   than the threshold value. */

#define CC1120_FIFOTHR_FIFO_THR_127_1     0x00
#define CC1120_FIFOTHR_FIFO_THR_126_2     0x01
#define CC1120_FIFOTHR_FIFO_THR_125_3     0x02
#define CC1120_FIFOTHR_FIFO_THR_124_4     0x03
#define CC1120_FIFOTHR_FIFO_THR_123_5     0x04
#define CC1120_FIFOTHR_FIFO_THR_122_6     0x05
#define CC1120_FIFOTHR_FIFO_THR_121_7     0x06
#define CC1120_FIFOTHR_FIFO_THR_120_8     0x07
#define CC1120_FIFOTHR_FIFO_THR_119_9     0x08
#define CC1120_FIFOTHR_FIFO_THR_118_10    0x09
#define CC1120_FIFOTHR_FIFO_THR_117_11    0x0A
#define CC1120_FIFOTHR_FIFO_THR_116_12    0x0B
#define CC1120_FIFOTHR_FIFO_THR_115_13    0x0C
#define CC1120_FIFOTHR_FIFO_THR_114_14    0x0D
#define CC1120_FIFOTHR_FIFO_THR_113_15    0x0E
#define CC1120_FIFOTHR_FIFO_THR_112_16    0x0F
#define CC1120_FIFOTHR_FIFO_THR_111_17    0x10
#define CC1120_FIFOTHR_FIFO_THR_110_18    0x11
#define CC1120_FIFOTHR_FIFO_THR_109_19    0x12
#define CC1120_FIFOTHR_FIFO_THR_108_20    0x13
#define CC1120_FIFOTHR_FIFO_THR_107_21    0x14
#define CC1120_FIFOTHR_FIFO_THR_106_22    0x15
#define CC1120_FIFOTHR_FIFO_THR_105_23    0x16
#define CC1120_FIFOTHR_FIFO_THR_104_24    0x17
#define CC1120_FIFOTHR_FIFO_THR_103_25    0x18
#define CC1120_FIFOTHR_FIFO_THR_102_26    0x19
#define CC1120_FIFOTHR_FIFO_THR_101_27    0x1A
#define CC1120_FIFOTHR_FIFO_THR_100_28    0x1B
#define CC1120_FIFOTHR_FIFO_THR_99_29     0x1C
#define CC1120_FIFOTHR_FIFO_THR_98_30     0x1D
#define CC1120_FIFOTHR_FIFO_THR_97_31     0x1E
#define CC1120_FIFOTHR_FIFO_THR_96_32     0x1F
#define CC1120_FIFOTHR_FIFO_THR_95_33     0x20
#define CC1120_FIFOTHR_FIFO_THR_94_34     0x21
#define CC1120_FIFOTHR_FIFO_THR_93_35     0x22
#define CC1120_FIFOTHR_FIFO_THR_92_36     0x23
#define CC1120_FIFOTHR_FIFO_THR_91_37     0x24
#define CC1120_FIFOTHR_FIFO_THR_90_38     0x25
#define CC1120_FIFOTHR_FIFO_THR_89_39     0x26
#define CC1120_FIFOTHR_FIFO_THR_88_40     0x27
#define CC1120_FIFOTHR_FIFO_THR_87_41     0x28
#define CC1120_FIFOTHR_FIFO_THR_86_42     0x29
#define CC1120_FIFOTHR_FIFO_THR_85_43     0x2A
#define CC1120_FIFOTHR_FIFO_THR_84_44     0x2B
#define CC1120_FIFOTHR_FIFO_THR_83_45     0x2C
#define CC1120_FIFOTHR_FIFO_THR_82_46     0x2D
#define CC1120_FIFOTHR_FIFO_THR_81_47     0x2E
#define CC1120_FIFOTHR_FIFO_THR_80_48     0x2F
#define CC1120_FIFOTHR_FIFO_THR_79_49     0x30
#define CC1120_FIFOTHR_FIFO_THR_78_50     0x31
#define CC1120_FIFOTHR_FIFO_THR_77_51     0x32
#define CC1120_FIFOTHR_FIFO_THR_76_52     0x33
#define CC1120_FIFOTHR_FIFO_THR_75_53     0x34
#define CC1120_FIFOTHR_FIFO_THR_74_54     0x35
#define CC1120_FIFOTHR_FIFO_THR_73_55     0x36
#define CC1120_FIFOTHR_FIFO_THR_72_56     0x37
#define CC1120_FIFOTHR_FIFO_THR_71_57     0x38
#define CC1120_FIFOTHR_FIFO_THR_70_58     0x39
#define CC1120_FIFOTHR_FIFO_THR_69_59     0x3A
#define CC1120_FIFOTHR_FIFO_THR_68_60     0x3B
#define CC1120_FIFOTHR_FIFO_THR_67_61     0x3C
#define CC1120_FIFOTHR_FIFO_THR_66_62     0x3D
#define CC1120_FIFOTHR_FIFO_THR_65_63     0x3E
#define CC1120_FIFOTHR_FIFO_THR_64_64     0x3F
#define CC1120_FIFOTHR_FIFO_THR_63_65     0x40
#define CC1120_FIFOTHR_FIFO_THR_62_66     0x41
#define CC1120_FIFOTHR_FIFO_THR_61_67     0x42
#define CC1120_FIFOTHR_FIFO_THR_60_68     0x43
#define CC1120_FIFOTHR_FIFO_THR_59_69     0x44
#define CC1120_FIFOTHR_FIFO_THR_58_70     0x45
#define CC1120_FIFOTHR_FIFO_THR_57_71     0x46
#define CC1120_FIFOTHR_FIFO_THR_56_72     0x47
#define CC1120_FIFOTHR_FIFO_THR_55_73     0x48
#define CC1120_FIFOTHR_FIFO_THR_54_74     0x49
#define CC1120_FIFOTHR_FIFO_THR_53_75     0x4A
#define CC1120_FIFOTHR_FIFO_THR_52_76     0x4B
#define CC1120_FIFOTHR_FIFO_THR_51_77     0x4C
#define CC1120_FIFOTHR_FIFO_THR_50_78     0x4D
#define CC1120_FIFOTHR_FIFO_THR_49_79     0x4E
#define CC1120_FIFOTHR_FIFO_THR_48_80     0x4F
#define CC1120_FIFOTHR_FIFO_THR_47_81     0x50
#define CC1120_FIFOTHR_FIFO_THR_46_82     0x51
#define CC1120_FIFOTHR_FIFO_THR_45_83     0x52
#define CC1120_FIFOTHR_FIFO_THR_44_84     0x53
#define CC1120_FIFOTHR_FIFO_THR_43_85     0x54
#define CC1120_FIFOTHR_FIFO_THR_42_86     0x55
#define CC1120_FIFOTHR_FIFO_THR_41_87     0x56
#define CC1120_FIFOTHR_FIFO_THR_40_88     0x57
#define CC1120_FIFOTHR_FIFO_THR_39_89     0x58
#define CC1120_FIFOTHR_FIFO_THR_38_90     0x59
#define CC1120_FIFOTHR_FIFO_THR_37_91     0x5A
#define CC1120_FIFOTHR_FIFO_THR_36_92     0x5B
#define CC1120_FIFOTHR_FIFO_THR_35_93     0x5C
#define CC1120_FIFOTHR_FIFO_THR_34_94     0x5D
#define CC1120_FIFOTHR_FIFO_THR_33_95     0x5E
#define CC1120_FIFOTHR_FIFO_THR_32_96     0x5F
#define CC1120_FIFOTHR_FIFO_THR_31_97     0x60
#define CC1120_FIFOTHR_FIFO_THR_30_98     0x61
#define CC1120_FIFOTHR_FIFO_THR_29_99     0x62
#define CC1120_FIFOTHR_FIFO_THR_28_100    0x63
#define CC1120_FIFOTHR_FIFO_THR_27_101    0x64
#define CC1120_FIFOTHR_FIFO_THR_26_102    0x65
#define CC1120_FIFOTHR_FIFO_THR_25_103    0x66
#define CC1120_FIFOTHR_FIFO_THR_24_104    0x67
#define CC1120_FIFOTHR_FIFO_THR_23_105    0x68
#define CC1120_FIFOTHR_FIFO_THR_22_106    0x69
#define CC1120_FIFOTHR_FIFO_THR_21_107    0x6A
#define CC1120_FIFOTHR_FIFO_THR_20_108    0x6B
#define CC1120_FIFOTHR_FIFO_THR_19_109    0x6C
#define CC1120_FIFOTHR_FIFO_THR_18_110    0x6D
#define CC1120_FIFOTHR_FIFO_THR_17_111    0x6E
#define CC1120_FIFOTHR_FIFO_THR_16_112    0x6F
#define CC1120_FIFOTHR_FIFO_THR_15_113    0x70
#define CC1120_FIFOTHR_FIFO_THR_14_114    0x71
#define CC1120_FIFOTHR_FIFO_THR_13_115    0x72
#define CC1120_FIFOTHR_FIFO_THR_12_116    0x73
#define CC1120_FIFOTHR_FIFO_THR_11_117    0x74
#define CC1120_FIFOTHR_FIFO_THR_10_118    0x75
#define CC1120_FIFOTHR_FIFO_THR_9_119     0x76
#define CC1120_FIFOTHR_FIFO_THR_8_120     0x77
#define CC1120_FIFOTHR_FIFO_THR_7_121     0x78
#define CC1120_FIFOTHR_FIFO_THR_6_122     0x79
#define CC1120_FIFOTHR_FIFO_THR_5_123     0x7A
#define CC1120_FIFOTHR_FIFO_THR_4_124     0x7B
#define CC1120_FIFOTHR_FIFO_THR_3_125     0x7C
#define CC1120_FIFOTHR_FIFO_THR_2_126     0x7D
#define CC1120_FIFOTHR_FIFO_THR_1_127     0x7E
#define CC1120_FIFOTHR_FIFO_THR_0_128     0x7F


/* DATA FIFO Access */
#define CC112X_SINGLE_TXFIFO            0x003F      /*  TXFIFO  - Single accecss to Transmit FIFO */
#define CC112X_BURST_TXFIFO             0x007F      /*  TXFIFO  - Burst accecss to Transmit FIFO  */
#define CC112X_SINGLE_RXFIFO            0x00BF      /*  RXFIFO  - Single accecss to Receive FIFO  */
#define CC112X_BURST_RXFIFO             0x00FF      /*  RXFIFO  - Burst ccecss to Receive FIFO  */

#define CC112X_LQI_CRC_OK_BM            0x80
#define CC112X_LQI_EST_BM               0x7F



/* Command strobe registers */
#define CC112X_SRES                     0x30      /*  SRES    - Reset chip. */
#define CC112X_SFSTXON                  0x31      /*  SFSTXON - Enable and calibrate frequency synthesizer. */
#define CC112X_SXOFF                    0x32      /*  SXOFF   - Turn off crystal oscillator. */
#define CC112X_SCAL                     0x33      /*  SCAL    - Calibrate frequency synthesizer and turn it off. */
#define CC112X_SRX                      0x34      /*  SRX     - Enable RX. Perform calibration if enabled. */
#define CC112X_STX                      0x35      /*  STX     - Enable TX. If in RX state, only enable TX if CCA passes. */
#define CC112X_SIDLE                    0x36      /*  SIDLE   - Exit RX / TX, turn off frequency synthesizer. */
#define CC112X_SWOR                     0x38      /*  SWOR    - Start automatic RX polling sequence (Wake-on-Radio) */
#define CC112X_SPWD                     0x39      /*  SPWD    - Enter power down mode when CSn goes high. */
#define CC112X_SFRX                     0x3A      /*  SFRX    - Flush the RX FIFO buffer. */
#define CC112X_SFTX                     0x3B      /*  SFTX    - Flush the TX FIFO buffer. */
#define CC112X_SWORRST                  0x3C      /*  SWORRST - Reset real time clock. */
#define CC112X_SNOP                     0x3D      /*  SNOP    - No operation. Returns status byte. */
#define CC112X_AFC                      0x37      /*  AFC     - Automatic Frequency Correction */

/* Chip states returned in status byte */
#define CC112X_STATE_IDLE               0x00
#define CC112X_STATE_RX                 0x10
#define CC112X_STATE_TX                 0x20
#define CC112X_STATE_FSTXON             0x30
#define CC112X_STATE_CALIBRATE          0x40
#define CC112X_STATE_SETTLING           0x50
#define CC112X_STATE_RXFIFO_ERROR       0x60
#define CC112X_STATE_TXFIFO_ERROR       0x70


/*========================= CHIP STATUS BYTE =================================*/
#define CC112X_STATUS_CHIP_RDY                 0x80
#define STATUS_CHIP_RDYn_BM             	0x80
#define CC112X_STATUS_STATE_BM                 0x70
#define CC112X_STATUS_FIFO_BYTES_AVAILABLE_BM  0x0F



/******************************************************************************
 * MACROS
 */

/* Macros for Tranceivers(TRX); copied from EasyLink on TRXEM Board */
#define st(x)      x
#define NOP()  __no_operation()

#define SPI_BEGIN()              st( SPI_PORT_OUT &= ~SPI_SPI_SC_N_PIN; NOP(); )
#define SPI_TX(x)                st( UCA1IFG &= ~UCRXIFG; UCA1TXBUF= (x); )
#define SPI_WAIT_DONE()          st( while(!(UCA1IFG & UCRXIFG)); )
#define SPI_RX()                 UCA1RXBUF
#define SPI_WAIT_MISO_LOW(x)     st( uint8_t count = 200; \
                                           while(SPI_PORT_IN & SPI_SPI_MISO_PIN) \
                                           { \
                                              __delay_cycles(5000); \
                                              count--; \
                                              if (count == 0) break; \
                                           } \
                                           if(count>0) (x) = 1; \
                                           else (x) = 0; )

#define SPI_END()                st( NOP(); SPI_PORT_OUT |= SPI_SPI_SC_N_PIN; )


/******************************************************************************
 * PROTOTYPES
 */ 

/* basic set of access functions */
rfStatus_t cc112xSpiReadReg(uint16_t addr, volatile uint8_t *data, uint8_t len);
rfStatus_t cc112xGetTxStatus(void);
rfStatus_t cc112xGetRxStatus(void);  
rfStatus_t cc112xSpiWriteReg(uint16_t addr, uint8_t *data, uint8_t len);
rfStatus_t cc112xSpiWriteTxFifo(uint8_t *pWriteData, uint8_t len);
rfStatus_t cc112xSpiReadRxFifo(uint8_t *pReadData, uint8_t len);
void ManualCalibration(void);
void TI_CC_Reset(void);
void SPI_Config();

#ifdef  __cplusplus
}
#endif
#endif// CC112x_SPI_H
